@@ -1,8 +1,6 @@
 package com.tildemh.debbuggtk;
 
 import org.gnu.gtk.*;
-import org.gnu.gtk.event.*;
-import java.util.LinkedList;
 
 /**
  * <p>Details of a column in the listingReport
