@@ -20,7 +20,7 @@ import org.gnu.glib.CustomEvents;
  * <p>This is released under the terms of the GNU Lesser General Public License
  * (LGPL). See the COPYING file for details.
  *
- * @version $Id: ListingReport.java,v 1.99 2004/04/21 20:20:05 mh Exp $
+ * @version $Id: ListingReport.java,v 1.100 2004/06/29 09:25:34 mh Exp $
  * @author &copy; Mark Howard &lt;mh@debian.org&gt; 2002
  */
 public class ListingReport extends VBox implements ListingListener{
@@ -44,6 +44,7 @@ public class ListingReport extends VBox implements ListingListener{
 	private long customEventTime = 0;
 	private LinkedList customEventAddQueue = new LinkedList();
 	private LinkedList customEventRemoveQueue = new LinkedList();
+	private volatile ListingState nextState = null;
 	
 	private ListStore store;
 	private TreeView view;
@@ -143,10 +144,37 @@ public class ListingReport extends VBox implements ListingListener{
 	DataColumnPixbuf dataStatus			= new DataColumnPixbuf();
 	DataColumnInt 	dataIntNum			= new DataColumnInt();
 	DataColumnInt 	dataIntSeverity 	= new DataColumnInt();
-	private void init(){
+	DataColumnString dataSortOrder = new DataColumnString();
+	private void init( boolean simpleInterface ){
 
 		makeView();
 
+
+		HBox filter = makeFilterBox();
+		if (!simpleInterface){
+			packStart( filter, false, false, 0);
+		}
+		packStart(sw);
+		if (simpleInterface){	
+		LinkedList cols = (LinkedList) columns.clone();
+		while (cols.size() > 0 ){
+			ListingColumn col = (ListingColumn) cols.removeFirst();
+			if ( col.getTitle().equals("Severity")
+					|| col.getTitle().equals("Title")
+					|| col.getTitle().equals("Date Created")
+			   )
+				col.setVisible( true );
+			else 
+				col.setVisible( false );
+		}
+		
+		changeColumns( columns );
+		}
+		setState(ListingState.IDLE);
+	}
+
+	private HBox makeFilterBox(){
+
 		HBox hb = new HBox(false, 0);
 		filter = new Entry();
 		hb.packStart( new Label( "Filter: "), false, false, 0);
@@ -194,11 +222,7 @@ public class ListingReport extends VBox implements ListingListener{
 			}
 		});
 		rcFilterBtn.setSensitive(false);
-
-		packStart(hb, false, false, 0);
-
-		packStart(sw);
-		setState(ListingState.IDLE);
+		return hb;
 	}
 
 	private void makeView( ){
@@ -234,7 +258,8 @@ public class ListingReport extends VBox implements ListingListener{
 			dataMoreinfo,
 			dataWontfix,
 			dataIntNum,
-			dataIntSeverity
+			dataIntSeverity,
+			dataSortOrder
 		});
 
 		view = new TreeView(store);
@@ -264,7 +289,6 @@ public class ListingReport extends VBox implements ListingListener{
 		colStatus.addAttributeMapping(statusRend, CellRendererPixbuf.Attribute.PIXBUF, dataStatus);
 		colStatus.setResizable(true);
 		colStatus.setReorderable(true);
-		colStatus.setSortColumn( dataFontWeight );
 
 		columns.add( new ListingColumn("Status", "", true, colStatus ) );
 		
@@ -379,6 +403,10 @@ public class ListingReport extends VBox implements ListingListener{
 		sw = new ScrolledWindow( new Adjustment(0,0,0,0,0,0), new Adjustment(0,0,0,0,0,0));
 		sw.setPolicy( PolicyType.AUTOMATIC, PolicyType.AUTOMATIC );
 		sw.add(view);
+		colStatus.setSortColumn( dataSortOrder ); //dataFontWeight );
+		colStatus.setSortIndicator( true );
+		colStatus.setSortOrder( SortType.ASCENDING );
+		store.setSortColumn(dataSortOrder, SortType.ASCENDING);
 	}
 
 	private void makeTextColumn( TreeViewColumn column, DataColumn data, String title, String example, boolean visible ){
@@ -405,6 +433,10 @@ public class ListingReport extends VBox implements ListingListener{
 		}
 	}
 
+	public LinkedList getColumns(){
+		return columns;
+	}
+
 	/**
 	 * Sets which columns should be displayed, and in what order.
 	 */
@@ -413,7 +445,8 @@ public class ListingReport extends VBox implements ListingListener{
 		setColumns();
 	}
 
-	public void showError(String txt){
+	public synchronized void showError( Listing list, String txt){
+		if (listing != list ) return;
 		MessageDialog md = 
 			new MessageDialog(parent, 
 						DialogFlags.MODAL.or(DialogFlags.DESTROY_WITH_PARENT), 
@@ -429,7 +462,7 @@ public class ListingReport extends VBox implements ListingListener{
 	////////////////////////////////////////////////////////////////////////////
 
 	public void editColumns(){
-		ListingColumnEditor lc = new ListingColumnEditor( this, columns ); 
+		new ListingColumnEditor( this, columns ); 
 	}
 	
 	/**
@@ -475,12 +508,29 @@ public class ListingReport extends VBox implements ListingListener{
 		// just in case any remain...
 		store.clear();
 	}
+	
+	private volatile ListingStub nextListing = null;
 	/**
 	 * sets the package to show listings for
 	 */
-	public synchronized void setPackage(ListingStub listingStub){
+	public void setPackage(ListingStub listingStub){
 		if (DebbugGtk.DEBUG) System.out.println("ListingReport.setPackage( "+listingStub+")");
+		if (listing != null && listingStub.equals( (ListingStub) listing )) return;
+		nextListing = listingStub;
+		if (constructorThread != null){
+			while (constructorThread.isAlive()){
+				constructorThread.interrupt();
+				Thread.yield();
+				try{
+				Thread.sleep( 10 );
+				}catch(InterruptedException e){ //ignore
+				}
+			}
+		}
+		if (nextListing != listingStub) return;
 
+		synchronized(this){
+		
 		if (listing != null){
 
 			if ( listingStub.equals((ListingStub) listing) ){
@@ -490,6 +540,15 @@ public class ListingReport extends VBox implements ListingListener{
 			listing.removeListener( (ListingListener) this );
 
 		}
+		
+		synchronized( customEventAddQueue ){
+			customEventAddQueue.clear();
+		}
+		synchronized( customEventRemoveQueue ){
+			customEventRemoveQueue.clear();
+		}
+		nextState = null;
+	
 		clearDisplay();
 
 		listener.listingChanged( listingStub );
@@ -503,7 +562,14 @@ public class ListingReport extends VBox implements ListingListener{
 		if (listing.getComplete()){
 				if (DebbugGtk.DEBUG) System.out.println("Setpackage - listing has bugreports.");
 
-			constructorObject =  new Runnable() { public void run(){ listing.loadReports(); } };
+				final Listing flisting = listing;
+			constructorObject =  new Runnable() { 
+				public void run(){ 
+					setStateThread( flisting, ListingState.LOADING);
+					listing.loadReports(); 
+					setStateThread(flisting, ListingState.IDLE);
+				} 
+			};
 		}else{
 				if (DebbugGtk.DEBUG) System.out.println("Setpackage - listing is incomplete - downloading.");
 			// if listing not downloaded, download it
@@ -513,7 +579,10 @@ public class ListingReport extends VBox implements ListingListener{
 		constructorThread = new Thread( constructorObject );
 		constructorThread.setName( "ListingConstructor");
 		constructorThread.setDaemon(true);
-		constructorThread.start();
+		synchronized (this){
+			if (constructorThread != null) constructorThread.interrupt();
+			constructorThread.start();
+		}}
 	if (DebbugGtk.DEBUG) System.out.println("000");
 	}
 
@@ -529,10 +598,18 @@ public class ListingReport extends VBox implements ListingListener{
 
 	// To be called from constructor thread!
 	private void filterReports(){
+		ListingState s = ListingState.FILTERING;
+		setStateThread(listing, s);
 		if (listing != null){
 			Hashtable bugtable = listing.getBugs();
 			Object[] bugs = (bugtable == null) ? new Object[0] : bugtable.keySet().toArray();
+			s.setMax( bugs.length );
 			for (int i = 0; i < bugs.length; i ++){
+
+				s.setBug( ((Integer) bugs[i]).intValue() );
+				s.setCurrent( i );
+				setStateThread( listing, s );
+				
 				Bug bug = BTS.getInstance().getBug( (Integer) bugs[i] );
 
 				if (!iterList.containsKey( bugs[i] ) ){
@@ -545,6 +622,7 @@ public class ListingReport extends VBox implements ListingListener{
 				}
 			}
 		}
+		setStateThread(listing, ListingState.IDLE);
 		flushCustomEventQueues();
 	}
 	
@@ -589,6 +667,10 @@ public class ListingReport extends VBox implements ListingListener{
 		TreeIter iter = store.appendRow();
 		bug.setData( iter );
 		iterList.put( new Integer( bug.getNumber() ), iter );
+		setValues(iter, bug );
+	}
+	
+	private void setValues( TreeIter iter, Bug bug ){
 		store.setValue(iter, dataBugObject, (Object) bug);
 		store.setValue(iter, dataNumber, Integer.toString(bug.getNumber()) );
 		store.setValue(iter, dataPackage, bug.getPackageName() );
@@ -612,6 +694,7 @@ public class ListingReport extends VBox implements ListingListener{
 
 			store.setValue(iter, dataSeverity, severity );
 		}
+		store.setValue(iter, dataIntSeverity, bug.getSeverity().getValue() );
 		store.setValue(iter, dataCreated, dateFormat.format(new Date( bug.getCreated() )) );
 		
 		// fixme: use better text
@@ -650,24 +733,35 @@ public class ListingReport extends VBox implements ListingListener{
 		store.setValue(iter, dataComments, bug.getCommentCount() );
 		store.setValue(iter, dataUnreproducible, bug.getTagUnreproducible() ? "?" : "" );
 		store.setValue(iter, dataIntNum, bug.getNumber() );
-		store.setValue(iter, dataIntSeverity, bug.getSeverity().getValue() );
+		String read = " ";
 		if (status.equals(Status.NEW) || status.equals(Status.UPDATED)){
+			read = "A";
 			if (bug.getIsDone())
 				store.setValue(iter, dataStatus, UNREAD_DONE);
 			else
 				store.setValue(iter, dataStatus, UNREAD);
 		}else{
+			read = "Z";
 			if (bug.getIsDone())
 				store.setValue(iter, dataStatus, READ_DONE);
 			else
 				store.setValue(iter, dataStatus, READ);
 		}
+		store.setValue( iter, dataSortOrder, read + bug.getSeverity().getValue() +"   " + bug.getNumber());
 	}
 
 	public ListingState getState(){
 		return state;
 	}
 
+	public synchronized void setStateThread( Listing list, ListingState state ){
+		if (listing != list) return;
+		nextState = state;
+		if (System.currentTimeMillis() - customEventTime > 100){
+			flushCustomEventQueues();
+		}
+	}
+
 	public void setState( ListingState state ){
 		this.state = state;
 		filterBtn.setSensitive(true);
@@ -679,12 +773,21 @@ public class ListingReport extends VBox implements ListingListener{
 		listener.listingReportStateChanged( state );
 	}
 		
+	public ListingReport(BTS bts, Listing pkg, ListingReportListener listener, Window parent, boolean simpleInterface){
+		super( false, 0 );
+		this.bts = bts;
+		this.parent = parent;
+		this.listener = listener;
+		init( simpleInterface );
+		setPackage(pkg);
+	}
+	
 	public ListingReport(BTS bts, Listing pkg, ListingReportListener listener, Window parent){
 		super( false, 0 );
 		this.bts = bts;
 		this.parent = parent;
 		this.listener = listener;
-		init();
+		init( false );
 		setPackage(pkg);
 	}
 	
@@ -693,7 +796,14 @@ public class ListingReport extends VBox implements ListingListener{
 		this.parent = parent;
 		this.bts = BTS.getInstance();
 		this.listener = listener;
-		init();
+		init( false );
+	}
+	public ListingReport( ListingReportListener listener, Window parent, boolean simpleInterface){
+		super( false, 0 );
+		this.parent = parent;
+		this.bts = BTS.getInstance();
+		this.listener = listener;
+		init( simpleInterface );
 	}
 
 	public ListingReport(BTS bts, ListingReportListener listener, Window parent){
@@ -701,7 +811,7 @@ public class ListingReport extends VBox implements ListingListener{
 		this.parent = parent;
 		this.bts = bts;
 		this.listener = listener;
-		init();
+		init( false );
 	}
 
 	public void setStatusListener( ListingStatusListener statusListener ){
@@ -719,7 +829,8 @@ public class ListingReport extends VBox implements ListingListener{
 	 * time (e.g. when loading a listing from disk), it may not be called for
 	 * every change.
 	 */
-	public void bugCountsChanged( Listing listing ){
+	public synchronized void bugCountsChanged( Listing listing ){
+		if (this.listing != listing) return;
 		// we're not interested in this one
 	}
 
@@ -727,46 +838,34 @@ public class ListingReport extends VBox implements ListingListener{
 	 * Called when the listing is about to contact the server to perform an
 	 * update.
 	 */
-	public void listingUpdateStart( Listing listing ){
+	public synchronized void listingUpdateStart( Listing listing ){
+		if (this.listing != listing) return;
 		if (DebbugGtk.DEBUG) System.out.println("ListingReport - listingUpdateStart");
-		CustomEvents.addEvent( new Runnable(){
-			public void run(){
-				setState( ListingState.UPDATING );
-			}
-		});
+		setStateThread( listing, ListingState.UPDATING );
 	}
 
 	/**
 	 * Called when the listing has completed it's interactions with the server.
 	 */
-	public void listingUpdateDone( Listing listing ){
+	public synchronized void listingUpdateDone( Listing listing ){
+		if (this.listing != listing) return;
 		if (DebbugGtk.DEBUG) System.out.println("ListingReport - listingUpdateDone");
-		CustomEvents.addEvent( new Runnable(){
-			public void run(){
-				setState( ListingState.IDLE );
-			}
-		});
+		setStateThread( listing, ListingState.IDLE );
 	}
 
 	/**
 	 * Called occasionally when downloading a package listing. This may be used
 	 * as part of a progress screen. 
 	 */
-	public void downloadingListing( Listing listing ){
+	public synchronized void downloadingListing( Listing listing ){
+		if (this.listing != listing) return;
 		if (DebbugGtk.DEBUG) System.out.println("downloadinglisting event");
-		CustomEvents.addEvent( new Runnable(){
-			public void run(){
-				setState( ListingState.DLLISTING );
-			}
-		});
+		setStateThread( listing, ListingState.DLLISTING );
 	}
-	public void interpretingListing( Listing listing ){
+	public synchronized void interpretingListing( Listing listing ){
+		if (this.listing != listing) return;
 		if (DebbugGtk.DEBUG) System.out.println("downloadinglisting event");
-		CustomEvents.addEvent( new Runnable(){
-			public void run(){
-				setState( ListingState.INTERPRET_LISTING );
-			}
-		});
+		setStateThread( listing, ListingState.INTERPRET_LISTING );
 	}
 
 	/**
@@ -778,23 +877,30 @@ public class ListingReport extends VBox implements ListingListener{
 	 * number of bugs in the listing, merely the number of bugs which need
 	 * updating.
 	 */
-	public void downloadingListingBug( Listing listing, Integer bugNumber, int pending, int total){
+	public synchronized void downloadingListingBug( Listing listing, Integer bugNumber, int pending, int total){
+		if (this.listing != listing) return;
 		final ListingState s =  ListingState.REFRESHING;
 		s.setMax( total );
 		s.setBug( bugNumber.intValue() );
 		s.setCurrent( pending );
-		CustomEvents.addEvent( new Runnable(){
-			public void run(){
-				setState( s );
-			}
-		});
+		setStateThread( listing, s );
+	}
+
+	public synchronized void loadingListingBug( Listing listing, Integer bugNumber, int pending, int total){
+		if (this.listing != listing) return;
+		final ListingState s =  ListingState.LOADING;
+		s.setMax( total );
+		s.setBug( bugNumber.intValue() );
+		s.setCurrent( pending );
+		setStateThread( listing, s );
 	}
 
 	/**
 	 * Called when a bug is removed from the listing (e.g. when the server says
 	 * the bug has been fixed for a list of open reports)
 	 */
-	public void bugRemoved( Listing listing, Integer bugNumber ){
+	public synchronized void bugRemoved( Listing listing, Integer bugNumber ){
+		if (this.listing != listing) return;
 		
 		if (iterList.containsKey(bugNumber)){
 			
@@ -811,7 +917,8 @@ public class ListingReport extends VBox implements ListingListener{
 	/**
 	 * Called when a bug is added to the listing for the first time.
 	 */
-	public void bugAdded( Listing listing, Bug bug ){
+	public synchronized void bugAdded( Listing listing, Bug bug ){
+		if (this.listing != listing) return;
 		if (DebbugGtk.DEBUG) System.out.println("Bug Added event - ListingReport");
 		if (!filterer.acceptable(bug))
 			return;
@@ -819,13 +926,13 @@ public class ListingReport extends VBox implements ListingListener{
 		synchronized( customEventAddQueue ){
 			customEventAddQueue.addLast( bug );
 		}
-
+		
 		if (System.currentTimeMillis() - customEventTime > 100){
 			flushCustomEventQueues();
 		}
 	}
 	
-	private void flushCustomEventQueues(){
+	private synchronized void flushCustomEventQueues(){
 		final Object[] addListCopy;
 		final Object[] removeListCopy;
 		synchronized( customEventAddQueue ){
@@ -836,20 +943,28 @@ public class ListingReport extends VBox implements ListingListener{
 			removeListCopy = customEventRemoveQueue.toArray();
 			customEventRemoveQueue.clear();
 		}
+		final Object lock = this;
+		final Listing finalListing = listing;
 		CustomEvents.addEvent( new Runnable(){
 			public void run(){
-				for (int i = 0; i < addListCopy.length; i++){
-					bugAddedHandler( (Bug) addListCopy[i] );
-				}
-				for (int i = 0; i < removeListCopy.length; i++){
-					bugRemovedHandler( (Integer) removeListCopy[i] );
+				synchronized(lock){
+					if (finalListing != listing) return;
+					if (nextState != null)
+						setState( nextState );
+					for (int i = 0; i < addListCopy.length; i++){
+						bugAddedHandler( (Bug) addListCopy[i] );
+					}
+					for (int i = 0; i < removeListCopy.length; i++){
+						bugRemovedHandler( (Integer) removeListCopy[i] );
+					}
 				}
 			}
 		});
 		customEventTime = System.currentTimeMillis();
 	}
 	
-	public void loadReportsDone(com.tildemh.debbug.Listing listing){
+	public synchronized void loadReportsDone(com.tildemh.debbug.Listing listing){
+		if (this.listing != listing) return;
 		flushCustomEventQueues();
 	}
 	
@@ -868,15 +983,21 @@ public class ListingReport extends VBox implements ListingListener{
 	 * Called when a modification is made to a bug in the listing (including
 	 * change of read/unread status) (excluding addition or removal of bugs).
 	 */
-	public void bugChanged( Listing listing, Bug bug ){
+	public synchronized void bugChanged( Listing listing, Bug bug ){
+		if (this.listing != listing) return;
 		if (DebbugGtk.DEBUG) System.out.println("ListingReport - BugChanged event");
 		Integer bugNum = new Integer( bug.getNumber() );
+		final Listing finalList = listing;
+		final ListingReport lock = this;
 		// previously acceptable bug is no longer acceptable.
 		if (!filterer.acceptable(bug) && iterList.containsKey(bugNum)){
 			final Integer bugNumber = bugNum;
 			CustomEvents.addEvent( new Runnable(){
 				public void run(){
-					store.removeRow( (TreeIter) iterList.get(bugNumber) );
+					synchronized(lock){
+						if (lock.listing == finalList)
+						store.removeRow( (TreeIter) iterList.get(bugNumber) );
+					}
 				}
 			});
 			return;
@@ -890,24 +1011,11 @@ public class ListingReport extends VBox implements ListingListener{
 		
 		// previously acceptable bug is still acceptable - record needs updating
 		if (filterer.acceptable(bug) && iterList.containsKey(bugNum)){
-			Status status =  bug.getStatus();
-			Severity severity = bug.getSeverity();
-			if ( status.equals(Status.NEW) || status.equals(Status.UPDATED) ){
-				if (iterList.containsKey(bugNum)){
-					store.setValue((TreeIter) iterList.get(bugNum) , dataFontWeight,  900);
-					store.setValue((TreeIter) iterList.get(bugNum) , dataStatus,  UNREAD);
-				}else{
-					System.out.println("*** ERROR - Bug not in iterList");
-					(new Exception()).printStackTrace();
-				}
+			if (iterList.containsKey(bugNum)){
+				setValues( (TreeIter) iterList.get(bugNum), bug );
 			}else{
-				if (iterList.containsKey(bugNum)){
-					store.setValue((TreeIter) iterList.get(bugNum) , dataFontWeight,  400);
-					store.setValue((TreeIter) iterList.get(bugNum) , dataStatus,  READ);
-				}else{
-					System.out.println("*** ERROR - Bug not in iterList");
-					(new Exception()).printStackTrace();
-				}
+				System.out.println("*** ERROR - Bug not in iterList");
+				(new Exception()).printStackTrace();
 			}
 			return;
 		}
@@ -920,12 +1028,14 @@ public class ListingReport extends VBox implements ListingListener{
 	 * Called when an exception occurred while updating the report. 
 	 * @return true if you want the execution to continue, false otherwise.
 	 */
-	public boolean listingException( Listing listing, Exception e ){
+	public synchronized boolean listingException( Listing listing, Exception e ){
+		if (this.listing != listing){ return false; } 
 		final Exception f = e;
+		setStateThread( listing, ListingState.ERROR );
+		final Listing flisting = listing;
 		CustomEvents.addEvent( new Runnable(){
 			public void run(){
-				setState( ListingState.ERROR );
-				showError( "listingException: "+f.toString() );
+				showError( flisting, "listingException: "+f.toString() );
 			}
 		});
 		return false;
